@@ -1,5 +1,6 @@
 from enum import Enum
 
+
 class ALConfiguration(Enum):
     NaiveBayesEstimator = "NaiveBayesEstimator"
     SVMEstimator = "SVMEstimator"
@@ -14,11 +15,22 @@ class ALConfiguration(Enum):
     RaschNBLRRFLGBM = "RaschNBLRRFLBGM"
     RaschNBLRRFLGBMRAND = "RaschNBLRRFLGBMRAND"
 
+
 class FEConfiguration(Enum):
     TFIDF5000 = "TfIDF5000"
 
+
 class EstimationConfiguration(str, Enum):
     RaschRidge = "RaschRidge"
     RaschParametric = "RaschParametric"
     RaschApproxParametric = "RaschApproxParametric"
-    RaschApproxConvParametric = "RaschApproxConvParametric"
\ No newline at end of file
+    RaschApproxConvParametric = "RaschApproxConvParametric"
+    CHAO = "Chao"
+
+
+class StopBuilderConfiguration(str, Enum):
+    CHAO_CONS_OPT = "Chao_CONS_OPT"
+
+
+class ExperimentCombination(str, Enum):
+    CHAO4 = "CHAO4"
